var app = angular.module("fireApp", ["firebase", "ngRoute"]);

// let's create a re-usable factory that generates the $firebaseAuth instance
app.factory("Auth", ["$firebaseAuth", 
  function($firebaseAuth) {
    var ref = new Firebase("https://sizzling-torch-4195.firebaseio.com/");
    return $firebaseAuth(ref); 
  }
]);

// this factory returns a synchronized array of messages
app.factory("Msgs", ["$firebaseArray",
  function($firebaseArray) {
    // create a reference to the database location where we will store our data
    var ref = new Firebase("https://sizzling-torch-4195.firebaseio.com/messages");

    // this uses AngularFire to create the synchronized array
    return $firebaseArray(ref);
  }
]);


app.controller("LoginCtrl", ["$scope", "$location", "Auth",
  function($scope, $location, Auth) {

    $scope.login = function() {
      Auth.$authWithPassword({
        email: $scope.email,
        password: $scope.password
      }).then(function(authData) {
        console.log("Logged in as:", authData.uid);
        $location.path("/home");
      }).catch(function(error) {
        console.error("Authentication failed:", error);
      });
    };

  }
]);


app.run(["$rootScope", "$location", function($rootScope, $location) {
  $rootScope.$on("$routeChangeError", function(event, next, previous, error) {
    // We can catch the error thrown when the $requireAuth promise is rejected
    // and redirect the user back to the home page
    if (error === "AUTH_REQUIRED") {
      $location.path("/");
    }
  });
}]);

app.config(["$routeProvider", function($routeProvider) {
  $routeProvider
  .when("/", {
    controller: "LoginCtrl",
    templateUrl: "views/auth.html"
  })
  .when("/home", {
    controller: "HomeCtrl",
    templateUrl: "views/home.html",
    resolve: {
      // controller will not be loaded until $requireAuth resolves
      // Auth refers to our $firebaseAuth wrapper in the example above
      "currentAuth": ["Auth", function(Auth) {
        // $requireAuth returns a promise so the resolve waits for it to complete
        // If the promise is rejected, it will throw a $stateChangeError (see above)
        return Auth.$requireAuth();
      }]
    }
  });
}]);

app.controller("HomeCtrl", ["currentAuth", "Auth", "Msgs", "$scope", "$location", 
  function(currentAuth, Auth, Msgs, $scope, $location) {
    
    // currentAuth (provided by resolve) will contain the
    // authenticated user or null if not logged in
    $scope.logout = function() {
      Auth.$unauth();
      $location.path("/");
    }

    // we add chatMessages array to the scope to be used in our ng-repeat
    $scope.messages = Msgs;

    // a method to create new messages; called by ng-submit
    $scope.addMessage = function() {
      // calling $add on a synchronized array is like Array.push(),
      // except that it saves the changes to our database!
      $scope.messages.$add({
        from: currentAuth.uid,
        content: $scope.message
      });

      // reset the message input
      $scope.message = "";
    };

  }
]);

app.controller("RegCtrl", ["currentAuth", "$scope", "Auth", 
  function($scope, Auth, currentAuth) {
    $scope.createUser = function() {
      $scope.message = null;
      $scope.error = null;

      Auth.$createUser({
        email: $scope.email,
        password: $scope.password
      }).then(function(userData) {
        $scope.message = "User created with uid: " + userData.uid;
      }).catch(function(error) {
        $scope.error = error;
      });
    };

    $scope.removeUser = function() {
      $scope.message = null;
      $scope.error = null;

      Auth.$removeUser({
        email: $scope.email,
        password: $scope.password
      }).then(function() {
        $scope.message = "User removed";
      }).catch(function(error) {
        $scope.error = error;
      });
    };
  }
]);

