var app = angular.module('myApp', ['firebase', 'ui.router', 'ui.bootstrap', 'ngAnimate']);

app.constant('dbref', 'https://sizzling-torch-4195.firebaseio.com');

// app.value('userId', 'uid');

app.factory('Auth', ['dbref', '$firebaseAuth', function(dbref, $firebaseAuth) {
  var fb = new Firebase(dbref);
  return $firebaseAuth(fb);
}]);

app.service('taskMgmt', ['$rootScope', '$firebaseArray', 'dbref', 'Auth', function($rootScope, $firebaseArray, dbref, Auth) {
  var current = {
    taskId  : '',
    title   : '',
    category: ''
  };

  var uid = Auth.$getAuth().uid;
  return {
    getTasks: function(category) {
      var ref = new Firebase(dbref)
        .child('tasks')
        .child(category);

      var tasks = $firebaseArray(ref);

      tasks.$loaded().then(function() {
        for (task in tasks) {
          if (tasks.hasOwnProperty(task)) {
            var match = false;
            for (key in tasks[task]['completed']) {
              if (tasks[task]['completed'][key] == uid) {
                match = true;
              }
            }
            tasks[task]['completed'] = match;
          }
        }
      });
      return tasks;
    },

    getCurrent: function() {
      return current;
    },
    /*
    setCurrent: function(category, taskId, title, points) {
      current['category'] = category;
      current['taskId'] = taskId;
      current['title'] = title;
      current['points'] = points;
    },*/
    setCurrent: function(category, taskId, task) {
      current['category'] = category;
      current['taskId'] = taskId;
      current['title'] = task['title'];
      current['points'] = task['points'];

      $rootScope.tasktitle = task['title'];

      if (task['type'] == 'Article') {
        $rootScope.article = {
          url     : task['url'],
          source  : task['source'],
          headline: task['headline']
        }
      }
    },

    setComplete: function() {
      // Get current task meta from rootscope.      
      var category = $rootScope.category;
      var taskId = $rootScope.taskId;

      // Firebase ref.
      var ref = new Firebase(dbref)
        .child('tasks')
        .child(category)
        .child(taskId);

      // Push userId to 'completed' array.
      ref.child('completed').push(uid);
      
      // Increment task count.
      ref.child('count').once('value', function(data) {
        ref.child('count').set(data.val() + 1);
      });
    },

    calcPercentage: function(tpoints) {
      var upoints = $rootScope.user['points'];

      var percent = "";

      if (upoints >= 0 && upoints + tpoints < 500) {
        percent = Math.floor((tpoints / 500) * 100) + "% closer";
      }

      else if (upoints >= 500 && upoints + tpoints < 1000) {
        percent = Math.floor((tpoints / 500) * 100) + "% closer";
      }

      return percent;
    }
  };
}]);

app.service('pointsMgmt', ['dbref', 'Auth','$rootScope', 'timelineMgmt', 'taskMgmt', function(dbref, Auth, $rootScope, timelineMgmt, taskMgmt) {
  // Get userId from Firebase Auth service.
  var uid = Auth.$getAuth().uid;
  return {
    addPoints: function(value) {
      // Firebase ref.
      var ref = new Firebase(dbref)
        .child('users')
        .child(uid);

      // Get meta from rootscope.
      var alias = $rootScope.user['alias'];
      var category = $rootScope.category;

      // Update user's total points (once).
      ref.child('points').once('value', function(data) {
        var oldPoints = data.val();
        var newPoints = data.val() + value;
        ref.child('points').set(newPoints);

        if (oldPoints < 500 && newPoints >= 500) {
          $rootScope.messagebar = true;
          $rootScope.messagebartext = "You have reached a new level: Intermediate.";
          ref.child('level').set('Intermediate');
          timelineMgmt.addEventLevel(alias, 'Intermediate');
        } else if (oldPoints < 1000 && newPoints == 1000) {
          $rootScope.messagebar = true;
          $rootScope.messagebartext = "You have reached a new level: Expert.";
          ref.child('level').set('Expert');
          timelineMgmt.addEventLevel(alias, 'Expert');
        }
      });

      // Add to task count for the relevant category.
      ref.child('tasks/' + category).once('value', function(data) {
        ref.child('tasks/' + category).set(data.val() + 1);
      });

      // Do necessary upddates on task.
      taskMgmt.setComplete();
    }
  }
}]);

app.service('gameMgmt', ['dbref', 'Auth', function(dbref, Auth) {
  var uid = Auth.$getAuth().uid;

  return {
    levelup: function() {

    }
  }
}]);

app.service('timelineMgmt', ['dbref', function(dbref) {
  var ref = new Firebase(dbref).child('timeline');
  return {
    addEventStart: function(alias) {
      ref.push({
        user    : alias,
        time    : Date.now(),
        type    : 'start',
        title   : "Started playing",
        message : alias + " has joined. Welcome!"
      });
    },
    addEventBadge: function(alias, badge) {
      ref.push({
        user    : alias,
        time    : Date.now(),
        type    : 'achievement',
        title   : "New achievement",
        message : alias + " earned the " + badge + " badge. Congratulations!"
      });
    },
    addEventLevel: function(alias, level) {
      ref.push({
        user    : alias,
        time    : Date.now(),
        type    : 'level',
        title   : "Level up!",
        message : alias + " has reached a new level: " + level + ". Awesome!"
      });
    },
    addEventComplete: function(alias, category) {
      ref.push({
        user    : alias,
        time    : Date.now(),
        type    : 'complete',
        title   : "All tasks complete",
        message : alias + " finished all tasks in the " + category + " category. Impressive!"
      });
    },
    addEventChallenge: function(alias, defeated) {
      ref.push({
        user    : alias,
        time    : Date.now(),
        type    : 'challenge',
        title   : "Challenge winner!",
        message : alias + " defeated " + defeated + " in a colleague challenge. Good job!"
      });
    }
  }
}]);

/**
	Auth
*/

app.controller("LoginCtrl", ['$scope', '$location', 'Auth', function($scope, $location, Auth) {
  $scope.login = function() {
    Auth.$authWithPassword({
      email: $scope.email,
      password: $scope.password
    }).then(function(authData) {
      $location.path("/");
    }).catch(function(error) {
      $scope.error = error;
    });
  };
}]);


app.run(["$rootScope", "$state", "$stateParams", function($rootScope, $state, $stateParams) {
  $rootScope.$on("$stateChangeError", function(event, toState, toParams, fromState, fromParams, error) {
    // We can catch the error thrown when the $requireAuth promise is rejected
    // and redirect the user back to the home page
    if (error === "AUTH_REQUIRED") {
      $state.go("auth");
    }
  });
}]);

app.filter('randomf', function() {
  return function(items) {
    var randomized = [];


  }
})


app.controller("MainCtrl", function($state) {
  $state.transitionTo('app.categories');
});

app.config(function($stateProvider, $urlRouterProvider, $locationProvider) {

	$urlRouterProvider.otherwise('/categories');

	$stateProvider
	.state('auth', {
		url: '/login',
    templateUrl: 'html/auth.html'
	})
	.state('app', {
    url: '/',
    templateUrl: 'html/app.html',
		resolve: {
      "currentAuth": ["Auth", function(Auth) {
        return Auth.$requireAuth();
      }]
    }
	})
  .state('app.categories', {
    url: 'categories',
    templateUrl: 'views/home.html',
  })
  .state('app.passwords', {
    url: 'passwords',
    templateUrl: 'views/passwords.html',
    controller: 'PwCtrl'
  })
  .state('app.wifi', {
    url: 'wifi',
    templateUrl: 'views/wifi.html',
    controller: 'WifiCtrl'
  })
  .state('app.web', {
    url: 'web',
    templateUrl: 'views/web.html',
    controller: 'WebCtrl'
  })
  .state('app.storage', {
    url: 'storage',
    templateUrl: 'views/storage.html',
    //controller: 'StorageCtrl'
  })
  .state('app.mobile', {
    url: 'mobile',
    templateUrl: 'views/mobile.html',
    //controller: 'MobileCtrl'
  })
  .state('app.social', {
    url: 'social',
    templateUrl: 'views/social.html',
    controller: 'SocialCtrl'
  })
  .state('app.email', {
    url: 'email',
    templateUrl: 'views/email.html',
    controller: 'EmailCtrl'
  })
  .state('app.essentials', {
    url: 'essentials',
    templateUrl: 'views/essentials.html',
    //controller: 'EssCtrl'
  })
  .state('app.working-remotely', {
    url: 'working-remotely',
    //templateUrl: 'views/working-remotely.html'
  })
  .state('app.removable', {
    url: 'removable',
    templateUrl: 'views/removable.html',
    //controller: 'RmCtrl'
  }) /*
  .state('app.sharepoint', {
    url: 'sharepoint',
    templateUrl: 'views/sharepoint.html'
  })
  .state('app.citrix', {
    url: 'citrix',
    templateUrl: 'views/citrix.html'
  }) */
  .state('app.leaderboard', {
    url: 'leaderboard',
    templateUrl: 'views/leaderboard.html',
    controller: 'LbCtrl'
  })

	/* $locationProvider.html5Mode(true); */
});

